package com.link.core.basic.lock;

import com.alibaba.fastjson.JSON;
import com.link.base.user.model.CoreUser;
import com.link.core.basic.service.BasicServiceException;
import com.link.core.util.BeanTools;
import com.link.core.util.RedisUtil;
import com.link.core.util.UserUtil;
import com.link.core.util.redisclient.LinkRedisClient;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.TimeUnit;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.Signature;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.aspectj.lang.reflect.MethodSignature;
import org.redisson.api.RLock;
import org.redisson.api.RedissonClient;
import org.springframework.stereotype.Component;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;

@Component
@Aspect
public class LinkRedisLockAspect {
   private static final String REDIS_LOCK_KEY_MODEL = "%s:%s";
   private static final String KEY_PREFIX = "RC-";
   private static final String RESULT_MESSAGE = "当前数据正在被%s(工号：%s，时间为：%tF %tT)编辑，请稍后重试。";
   private static final String EXISTS_FLAG_LOCK = "_exists_flag_lock";
   private static final String ID = "id";
   private static final String COLON = ":";
   private static final String POINT = ".";
   private static final String COMMA = ",";

   @Pointcut("@annotation(com.link.core.basic.lock.LinkRedisLock)")
   public void pointcutMethod() {
   }

   @Around("pointcutMethod()")
   public Object aroundMethod(ProceedingJoinPoint point) throws Throwable {
      try {
         String value = null;
         int linkExpire = 600;
         boolean unLock = true;
         Signature signature = point.getSignature();
         boolean methodFlag = ((MethodSignature)signature).getMethod().isAnnotationPresent(LinkRedisLock.class);
         if (methodFlag) {
            String source = ((LinkRedisLock)((MethodSignature)signature).getMethod().getAnnotation(LinkRedisLock.class)).source().getSource();
            boolean checkStatus = ((LinkRedisLock)((MethodSignature)signature).getMethod().getAnnotation(LinkRedisLock.class)).checkStatus();
            value = ((LinkRedisLock)((MethodSignature)signature).getMethod().getAnnotation(LinkRedisLock.class)).value();
            linkExpire = ((LinkRedisLock)((MethodSignature)signature).getMethod().getAnnotation(LinkRedisLock.class)).expireTime();
            unLock = ((LinkRedisLock)((MethodSignature)signature).getMethod().getAnnotation(LinkRedisLock.class)).unLock();
            if (Boolean.FALSE.equals(this.checkParameter(checkStatus, source, value))) {
               throw new BasicServiceException("强制检验时前端传入类型的时候必须传入对应的值");
            }
         } else {
            Object targetObj = point.getTarget();
            boolean classFlag = targetObj.getClass().isAnnotationPresent(LinkRedisLock.class);
            if (classFlag) {
               linkExpire = ((LinkRedisLock)targetObj.getClass().getAnnotation(LinkRedisLock.class)).expireTime();
            }
         }

         Object[] params = point.getArgs();
         String redisLockKey = this.assembleRedisKey(params, value, point);
         Object result = this.addRedisLock(redisLockKey, linkExpire, point, unLock);
         return result;
      } catch (Exception var13) {
         Exception e = var13;
         e.printStackTrace();
         throw new BasicServiceException(e);
      }
   }

   private boolean checkParameter(boolean checkStatus, String source, String value) {
      boolean result = true;
      boolean sourceFlag = source.equals(LockSourceEnum.FRONT.getSource());
      if (checkStatus && sourceFlag && StringUtils.isEmpty(value)) {
         result = false;
      }

      return result;
   }

   private String assembleRedisKey(Object[] params, String value, ProceedingJoinPoint point) {
      Signature signature = point.getSignature();
      MethodSignature methodSignature = (MethodSignature)signature;
      String[] parameterNames = methodSignature.getParameterNames();
      Map<String, Object> paramsMap = new HashMap();
      StringBuilder resultKey = new StringBuilder();
      StringBuilder basicTypeKey = new StringBuilder();

      for(int i = 0; i < parameterNames.length; ++i) {
         String parameterName = parameterNames[i];
         Object paramObject = params[i];
         paramsMap.put(parameterName, paramObject);
      }

      String[] parameterNameArray;
      int var20;
      if (!StringUtils.isEmpty(value)) {
         parameterNameArray = value.split(",");
         String[] var18 = parameterNameArray;
         var20 = parameterNameArray.length;

         for(int var13 = 0; var13 < var20; ++var13) {
            String parameterNameStr = var18[var13];
            if (parameterNameStr.contains(".")) {
               String parameterName = parameterNameStr.substring(0, value.indexOf("."));
               String attributeName = parameterNameStr.substring(value.indexOf(".") + 1);
               resultKey.append(this.getResultKey(paramsMap, parameterName, attributeName, parameterNames, false));
            } else {
               basicTypeKey.append(this.getResultKey(paramsMap, parameterNameStr, (String)null, parameterNames, false));
            }
         }
      } else {
         parameterNameArray = parameterNames;
         int var19 = parameterNames.length;

         for(var20 = 0; var20 < var19; ++var20) {
            String parameterName = parameterNameArray[var20];
            resultKey.append(this.getResultKey(paramsMap, parameterName, "id", parameterNames, true));
         }
      }

      return "RC-" + basicTypeKey.append(resultKey);
   }

   private StringBuilder getResultKey(Map<String, Object> paramsMap, String parameterName, String attributeName, String[] parameterNames, boolean isValueEmpty) {
      StringBuilder objectTypeKey = new StringBuilder();
      int count = 0;
      int basicTypeCount = 0;
      String[] var9 = parameterNames;
      int var10 = parameterNames.length;

      for(int var11 = 0; var11 < var10; ++var11) {
         String name = var9[var11];
         ++count;
         Object paramObject;
         if (Boolean.FALSE.equals(isValueEmpty)) {
            paramObject = paramsMap.get(parameterName);
            if (StringUtils.isEmpty(attributeName) && name.equals(parameterName)) {
               if (Boolean.FALSE.equals(StringUtils.isEmpty(attributeName)) && name.equals(attributeName)) {
                  ++basicTypeCount;
               }

               if (basicTypeCount > 1) {
                  objectTypeKey.append(":");
               }

               objectTypeKey.append(paramObject);
            }
         } else {
            paramObject = paramsMap.get(name);
            if (!(paramObject instanceof HttpSession) && !(paramObject instanceof HttpServletRequest) && !(paramObject instanceof HttpServletResponse)) {
               this.generateResultKey(paramObject, "id", count, objectTypeKey);
            }
         }
      }

      return objectTypeKey;
   }

   private StringBuilder generateResultKey(Object paramObject, String attributeName, int count, StringBuilder objectTypeKey) {
      Map<String, Object> objectMap = JSON.parseObject(JSON.toJSONString(paramObject));
      String attributeValue = objectMap.get(attributeName).toString();
      if (objectMap.containsKey(attributeName)) {
         if (count > 1) {
            objectTypeKey.append(":");
         }

         objectTypeKey.append(String.format("%s:%s", attributeName, attributeValue));
      }

      return objectTypeKey;
   }

   private Object addRedisLock(String key, int linkExpire, ProceedingJoinPoint point, boolean unLock) throws Throwable {
      Object result = null;
      LinkRedisClient redis = null;
      RLock rlock = null;

      try {
         redis = RedisUtil.getJedis();
         RedissonClient redissonClient = (RedissonClient)BeanTools.getBean(RedissonClient.class);
         rlock = ((RedissonClient)Objects.requireNonNull(redissonClient)).getLock(key + "_exists_flag_lock");
         boolean lockFlag = ((RLock)Objects.requireNonNull(rlock)).tryLock(0L, (long)linkExpire, TimeUnit.SECONDS);
         if (!lockFlag) {
            boolean existsAgainFlag = redis.exists(key);
            if (existsAgainFlag) {
               throw new BasicServiceException(redis.get(key));
            }

            throw new BasicServiceException("获取锁失败！");
         }

         initLock(key, linkExpire, redis);
         result = point.proceed();
      } finally {
         if (unLock && !ObjectUtils.isEmpty(rlock) && rlock.isHeldByCurrentThread()) {
            rlock.unlock();
            ((LinkRedisClient)Objects.requireNonNull(redis)).del(key);
         }

         RedisUtil.returnResource(redis);
      }

      return result;
   }

   private static void initLock(String key, int linkExpire, LinkRedisClient redis) {
      CoreUser currentUser = UserUtil.getUser();
      Date date = new Date();
      String value = String.format("当前数据正在被%s(工号：%s，时间为：%tF %tT)编辑，请稍后重试。", currentUser.getFirstName(), currentUser.getUsername(), date, date);
      redis.set(key, value);
      redis.expire(key, linkExpire);
   }
}
